----------------------------------------------------------------------------------
--                                 _             _
--                                | |_  ___ _ __(_)__ _
--                                | ' \/ -_) '_ \ / _` |
--                                |_||_\___| .__/_\__,_|
--                                         |_|
--
----------------------------------------------------------------------------------
--
-- Company: hepia
-- Author: Joachim Schmidt <joachim.schmidt@hesge.ch>
--
-- Module Name: scalp_booking_vector.vhd
-- Target Device: SCALP xc7z015clg485-2
-- Tool version: 2019.1
-- Description: Manage the booking vector.
--
-- Last update: 2020-02-13
--
---------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--use ieee.std_logic_unsigned.all;
--use ieee.std_logic_arith.all;
--use ieee.std_logic_misc.all;

--library UNISIM;
--use UNISIM.VCOMPONENTS.all;

use work.scalp_misc.all;

entity scalp_booking_vector is

    generic (
        C_SCALP_INTERFACE_VECTOR_SIZE     : integer range 0 to 255 := 7;
        C_SCALP_NEIGHBORS_VECTOR_SIZE     : integer range 0 to 255 := 6;
        C_SCALP_SCHEDULER_ACK_VECTOR_SIZE : integer range 0 to 255 := 1);

    port (
        SysClkxCI             : in  std_ulogic;
        SysRstxRNAI           : in  std_ulogic;
        IfDstIdxxDI           : in  integer range 0 to (C_SCALP_INTERFACE_VECTOR_SIZE - 1);
        IfDstIdxValidxSI      : in  std_ulogic;
        SchedulerAckxDI       : in  std_ulogic_vector((C_SCALP_SCHEDULER_ACK_VECTOR_SIZE - 1) downto 0);
        BookingVectorxDO      : out std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
        BookingVectorValidxSO : out std_ulogic);

end entity scalp_booking_vector;

architecture behavioral of scalp_booking_vector is

    signal IfDstIdxValidxS            : std_ulogic                                                          := '0';
    signal IfDstIdxxD                 : integer range 0 to (C_SCALP_INTERFACE_VECTOR_SIZE - 1)              := 0;
    signal SchedulerAckxD             : std_ulogic_vector((C_SCALP_SCHEDULER_ACK_VECTOR_SIZE - 1) downto 0) := (others => '0');
    signal BookingVectorxDP           : std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0)     := (others => '0');
    signal BookingVectorxDN           : std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0)     := (others => '0');
    signal BookingVectorValidxS       : std_ulogic                                                          := '0';
    -- Booking vector state signals
    signal ScalpBookingVectorStatexDP : t_scalp_booking_vector_states                                       := E_SCALP_BOOKING_VECTOR_SCHEDULER_ACK;
    signal ScalpBookingVectorStatexDN : t_scalp_booking_vector_states                                       := E_SCALP_BOOKING_VECTOR_SCHEDULER_ACK;

begin  -- architecture behavioral

    EntityIOxB : block is
    begin  -- block EntityIOxB
        IfDstIdxxAS           : IfDstIdxxD            <= IfDstIdxxDI;
        IfDstIdxValidxAS      : IfDstIdxValidxS       <= IfDstIdxValidxSI;
        RXSchedulerAckxAS     : SchedulerAckxD        <= SchedulerAckxDI;
        TXBookingVectorxAS    : BookingVectorxDO      <= BookingVectorxDP;
        BookingVectorValidxAS : BookingVectorValidxSO <= BookingVectorValidxS;
    end block EntityIOxB;

    StateMachineUpdateRegxP : process (SysClkxCI, SysRstxRNAI) is
    begin  -- process StateMachineUpdateRegxP
        if SysRstxRNAI = '0' then
            BookingVectorxDP           <= (others => '0');
            ScalpBookingVectorStatexDP <= E_SCALP_BOOKING_VECTOR_SCHEDULER_ACK;
        elsif rising_edge(SysClkxCI) then
            BookingVectorxDP           <= BookingVectorxDN;
            ScalpBookingVectorStatexDP <= ScalpBookingVectorStatexDN;
        end if;
    end process StateMachineUpdateRegxP;

    BookingVectorStateMachinexP : process (BookingVectorxDP, IfDstIdxValidxS,
                                           IfDstIdxxD,
                                           ScalpBookingVectorStatexDP,
                                           SchedulerAckxD(0)) is
    begin  -- process BookingVectorStateMachinexP
        -- Default value
        BookingVectorxDN           <= BookingVectorxDP;
        ScalpBookingVectorStatexDN <= ScalpBookingVectorStatexDP;
        BookingVectorValidxS       <= '0';

        case ScalpBookingVectorStatexDP is
            when E_SCALP_BOOKING_VECTOR_SCHEDULER_ACK =>
                    if IfDstIdxValidxS = '1' then
                        BookingVectorxDN(IfDstIdxxD) <= '1';
                        ScalpBookingVectorStatexDN   <= E_SCALP_BOOKING_VECTOR_VALID_INDEX;
                        BookingVectorValidxS         <= '1';
                    end if;

            when E_SCALP_BOOKING_VECTOR_VALID_INDEX =>
                    BookingVectorValidxS <= '1';
                    -- !!! SchedulerAckxD(0) Only if the acknowledgment signals are multiplexed.
                    if SchedulerAckxD(0) = '1' then
                        BookingVectorxDN           <= (others => '0');
                        ScalpBookingVectorStatexDN <= E_SCALP_BOOKING_VECTOR_SCHEDULER_ACK;
                        BookingVectorValidxS       <= '0';
                    end if;

            when others => null;
        end case;
    end process BookingVectorStateMachinexP;

end architecture behavioral;
