----------------------------------------------------------------------------------
--                                 _             _
--                                | |_  ___ _ __(_)__ _
--                                | ' \/ -_) '_ \ / _` |
--                                |_||_\___| .__/_\__,_|
--                                         |_|
--
----------------------------------------------------------------------------------
--
-- Company: hepia
-- Author: Joachim Schmidt <joachim.schmidt@hesge.ch>
--
-- Module Name: scalp_router_core.vhd
-- Target Device: SCALP xc7z015clg485-2
-- Tool version: 2019.1
-- Description: Scalp Router Core Algorithm
--
-- Last update: 2020-01-21
--
---------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--use ieee.std_logic_unsigned.all;
--use ieee.std_logic_arith.all;
--use ieee.std_logic_misc.all;

--library UNISIM;
--use UNISIM.VCOMPONENTS.all;

use work.scalp_misc.all;

entity scalp_router_core is

    generic (
        C_SCALP_INTERFACE_VECTOR_SIZE : integer range 0 to 255 := 7);

    port (
        -- System Clock and Reset
        SysClkxCI             : in  std_ulogic;
        SysRstxRNAI           : in  std_ulogic;
        -- Local and Destination Router Network Address
        LocRouterNetAddrxDI   : in  t_scalp_netaddr;
        DstRouterNetAddrxDI   : in  t_scalp_netaddr;
        RouterNetAddrValidxSI : in  std_ulogic;
        -- Destination Interfaces Number
        IfDstNumxDO           : out integer range 0 to (C_SCALP_INTERFACE_VECTOR_SIZE - 1);
        IfNumValidxSO         : out std_ulogic;
        -- QoS Vector
        QoSVectorxDI          : in  t_scalp_qos_vector((C_SCALP_INTERFACE_VECTOR_SIZE - 1) downto 0));

end entity scalp_router_core;

architecture behavioral_xyz of scalp_router_core is

    constant C_SCALP_NORTH_NUM  : integer range 0 to (C_SCALP_INTERFACE_VECTOR_SIZE - 1) := 0;
    constant C_SCALP_EAST_NUM   : integer range 0 to (C_SCALP_INTERFACE_VECTOR_SIZE - 1) := 1;
    constant C_SCALP_SOUTH_NUM  : integer range 0 to (C_SCALP_INTERFACE_VECTOR_SIZE - 1) := 2;
    constant C_SCALP_WEST_NUM   : integer range 0 to (C_SCALP_INTERFACE_VECTOR_SIZE - 1) := 3;
    constant C_SCALP_TOP_NUM    : integer range 0 to (C_SCALP_INTERFACE_VECTOR_SIZE - 1) := 4;
    constant C_SCALP_BOTTOM_NUM : integer range 0 to (C_SCALP_INTERFACE_VECTOR_SIZE - 1) := 5;
    constant C_SCALP_LOCAL_NUM  : integer range 0 to (C_SCALP_INTERFACE_VECTOR_SIZE - 1) := 6;

    signal LocRouterNetAddrxD   : t_scalp_netaddr                                                  := C_3D_MIN_SCALP_NETADDR;
    signal DstRouterNetAddrxD   : t_scalp_netaddr                                                  := C_3D_MIN_SCALP_NETADDR;
    signal RouterNetAddrValidxS : std_ulogic                                                       := '0';
    signal IfDstNumxD           : integer range 0 to (C_SCALP_INTERFACE_VECTOR_SIZE - 1)           := 0;
    signal IfNumValidxS         : std_ulogic                                                       := '0';
    signal QoSVectorxD          : t_scalp_qos_vector((C_SCALP_INTERFACE_VECTOR_SIZE - 1) downto 0) := (others => C_SCALP_NO_QOS);

begin  -- architecture behavioral

    EntityIOxB : block is
    begin  -- block EntityIOxB
        LocRouterNetAddrxAS   : LocRouterNetAddrxD   <= LocRouterNetAddrxDI;
        DstRouterNetAddrxAS   : DstRouterNetAddrxD   <= DstRouterNetAddrxDI;
        RouterNetAddrValidxAS : RouterNetAddrValidxS <= RouterNetAddrValidxSI;
        IfDstNumxAS           : IfDstNumxDO          <= IfDstNumxD;
        IfNumValidxAS         : IfNumValidxSO        <= IfNumValidxS;
        QoSVectorxAS          : QoSVectorxD          <= QoSVectorxDI;
    end block EntityIOxB;

    ScalpRouterCorexP : process (SysClkxCI, SysRstxRNAI) is
    begin  -- process ScalpRouterCorexP
        if SysRstxRNAI = '0' then
            IfDstNumxD   <= C_SCALP_NORTH_NUM;
            IfNumValidxS <= '0';
        elsif rising_edge(SysClkxCI) then
            IfDstNumxD   <= C_SCALP_NORTH_NUM;
            IfNumValidxS <= '0';

            if RouterNetAddrValidxS = '1' then
                IfNumValidxS <= '1';
                -- X-axis
                if LocRouterNetAddrxD.XxD < DstRouterNetAddrxD.XxD then
                    IfDstNumxD <= C_SCALP_EAST_NUM;
                elsif LocRouterNetAddrxD.XxD > DstRouterNetAddrxD.XxD then
                    IfDstNumxD <= C_SCALP_WEST_NUM;
                else
                    -- Y-axis
                    if LocRouterNetAddrxD.YxD < DstRouterNetAddrxD.YxD then
                        IfDstNumxD <= C_SCALP_NORTH_NUM;
                    elsif LocRouterNetAddrxD.YxD > DstRouterNetAddrxD.YxD then
                        IfDstNumxD <= C_SCALP_SOUTH_NUM;
                    else
                        -- Z-axis
                        if LocRouterNetAddrxD.ZxD < DstRouterNetAddrxD.ZxD then
                            IfDstNumxD <= C_SCALP_TOP_NUM;
                        elsif LocRouterNetAddrxD.ZxD > DstRouterNetAddrxD.ZxD then
                            IfDstNumxD <= C_SCALP_BOTTOM_NUM;
                        else
                            IfDstNumxD <= C_SCALP_LOCAL_NUM;
                        end if;
                    end if;
                end if;
            end if;
        end if;
    end process ScalpRouterCorexP;

end architecture behavioral_xyz;
