----------------------------------------------------------------------------------
--                                 _             _
--                                | |_  ___ _ __(_)__ _
--                                | ' \/ -_) '_ \ / _` |
--                                |_||_\___| .__/_\__,_|
--                                         |_|
--
----------------------------------------------------------------------------------
--
-- Company: hepia
-- Author: Joachim Schmidt <joachim.schmidt@hesge.ch>
--
-- Module Name: scalp_router_interface.vhd
-- Target Device: SCALP xc7z015clg485-2
-- Tool version: 2019.1
-- Description: Scalp Router Interface.
--
-- Last update: 2020-11-30
--
---------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

--use ieee.std_logic_unsigned.all;
--use ieee.std_logic_arith.all;
--use ieee.std_logic_misc.all;

--library UNISIM;
--use UNISIM.VCOMPONENTS.all;

use work.axi4_pkg.all;
use work.scalp_misc.all;

entity scalp_router_interface is

    generic (
        C_SCALP_INTERFACE_VECTOR_SIZE     : integer range 0 to 255 := 7;
        C_SCALP_INTERFACE_SOURCE_NUMBER   : integer range 0 to 255 := 0;
        C_SCALP_NEIGHBORS_VECTOR_SIZE     : integer range 0 to 255 := 6;
        C_SCALP_SCHEDULER_ACK_VECTOR_SIZE : integer range 0 to 255 := 1;
        C_SCALP_SCHEDULER_STRATEGY        : string                 := "RR");

    port (
        -- System Clock and System nReset
        SysClkxCI         : in  std_ulogic;
        SysRstxRNAI       : in  std_ulogic;
        -- Local Router Network Address
        LocNetAddrxDI     : in  t_scalp_netaddr;
        -- Axi4 Stream
        -----------------------------------------------------------------------
        -- Interface Input Side
        RXAxi4m2sIfxDI    : in  t_axi4m2s;
        RXAxi4s2mIfxDO    : out t_axi4s2m;
        -- Interface Output Side
        TXAxi4m2sIfxDO    : out t_axi4m2s;
        TXAxi4s2mIfxDI    : in  t_axi4s2m;
        -----------------------------------------------------------------------
        -- Neighborhood Links Input Side
        RXAxi4m2sLinksxDI : in  t_axi4m2s_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
        RXAxi4s2mLinksxDO : out t_axi4s2m_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
        -- Neighborhood Links Output Side
        TXAxi4m2sLinksxDO : out t_axi4m2s_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
        TXAxi4s2mLinksxDI : in  t_axi4s2m_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
        -----------------------------------------------------------------------
        -- Booking Vector
        -- Input Side
        BookingVectorxDI  : in  std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
        -- Output Side
        BookingVectorxDO  : out std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
        -- Scheduler Ack Vector
        -- Input Side
        SchedulerAckxDI   : in  std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
        -- Output Side
        SchedulerAckxDO   : out std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
        -- QoS Vector
        QoSVectorxDI      : in  t_scalp_qos_vector((C_SCALP_INTERFACE_VECTOR_SIZE - 1) downto 0));

end entity scalp_router_interface;

architecture rtl of scalp_router_interface is

    -- Scalp RX Side
    constant C_SCALP_SCHEDULER_OUTPUT_REG : boolean                := true;
    constant C_SCALP_DELAY_SIZE           : integer range 0 to 255 := 2;

    component scalp_tx_side is
        generic (
            C_SCALP_SCHEDULER_ACK_VECTOR_SIZE : integer range 0 to 255);
        port (
            SysClkxCI             : in  std_ulogic;
            SysRstxRNAI           : in  std_ulogic;
            DstRouterNetAddrxDO   : out t_scalp_netaddr;
            RouterNetAddrValidxSO : out std_ulogic;
            BookingVectorValidxSI : in  std_ulogic;
            -- SchedulerAckxDI       : in  std_ulogic_vector((C_SCALP_SCHEDULER_ACK_VECTOR_SIZE - 1) downto 0);
            RXAxi4M2SLinkxDI      : in  t_axi4m2s;
            RXAxi4S2MLinkxDO      : out t_axi4s2m;
            TXAxi4M2SLinkxDO      : out t_axi4m2s;
            TXAxi4S2MLinkxDI      : in  t_axi4s2m);
    end component scalp_tx_side;

    component scalp_rx_side is
        generic (
            C_SCALP_NEIGHBORS_VECTOR_SIZE : integer range 0 to 255;
            C_SCALP_SCHEDULER_STRATEGY    : string;
            C_SCALP_SCHEDULER_OUTPUT_REG  : boolean);
        port (
            SysClkxCI            : in  std_ulogic;
            SysRstxRNAI          : in  std_ulogic;
            SchedulerAckxDO      : out std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
            BookingVectorxDI     : in  std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
            TXAxi4M2SLinkxDO     : out t_axi4m2s;
            TXAxi4S2MLinkxDI     : in  t_axi4s2m;
            RXAxi4M2SLinkxDI     : in  t_axi4m2s_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
            RXAxi4S2MLinkxDO     : out t_axi4s2m_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
            BookingIndexxDO      : out integer range 0 to (C_SCALP_NEIGHBORS_VECTOR_SIZE - 1);
            BookingIndexValidxSO : out std_ulogic);
    end component scalp_rx_side;

    component scalp_router_core is
        generic (
            C_SCALP_INTERFACE_VECTOR_SIZE : integer range 0 to 255);
        port (
            SysClkxCI             : in  std_ulogic;
            SysRstxRNAI           : in  std_ulogic;
            LocRouterNetAddrxDI   : in  t_scalp_netaddr;
            DstRouterNetAddrxDI   : in  t_scalp_netaddr;
            RouterNetAddrValidxSI : in  std_ulogic;
            IfDstNumxDO           : out integer range 0 to (C_SCALP_INTERFACE_VECTOR_SIZE - 1);
            IfNumValidxSO         : out std_ulogic;
            QoSVectorxDI          : in  std_ulogic_vector((C_SCALP_INTERFACE_VECTOR_SIZE - 1) downto 0));
    end component scalp_router_core;

    component scalp_scheduler_ack is
        generic (
            C_SCALP_NEIGHBORS_VECTOR_SIZE     : integer range 0 to 255;
            C_SCALP_SCHEDULER_ACK_VECTOR_SIZE : integer range 0 to 255);
        port (
            SchedulerAckxDI : in  std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
            SchedulerAckxDO : out std_ulogic_vector((C_SCALP_SCHEDULER_ACK_VECTOR_SIZE - 1) downto 0));
    end component scalp_scheduler_ack;

    component scalp_matrix_index is
        generic (
            C_SCALP_INTERFACE_VECTOR_SIZE : integer range 0 to 255);
        port (
            IfSrcNumxDI      : in  integer range 0 to (C_SCALP_INTERFACE_VECTOR_SIZE - 1);
            IfDstNumxDI      : in  integer range 0 to (C_SCALP_INTERFACE_VECTOR_SIZE - 1);
            IfNumValidxSI    : in  std_ulogic;
            IfDstIdxxDO      : out integer range 0 to (C_SCALP_INTERFACE_VECTOR_SIZE - 1);
            IfDstIdxValidxSO : out std_ulogic);
    end component scalp_matrix_index;

    component scalp_booking_vector is
        generic (
            C_SCALP_INTERFACE_VECTOR_SIZE     : integer range 0 to 255;
            C_SCALP_NEIGHBORS_VECTOR_SIZE     : integer range 0 to 255;
            C_SCALP_SCHEDULER_ACK_VECTOR_SIZE : integer range 0 to 255);
        port (
            SysClkxCI             : in  std_ulogic;
            SysRstxRNAI           : in  std_ulogic;
            IfDstIdxxDI           : in  integer range 0 to (C_SCALP_INTERFACE_VECTOR_SIZE - 1);
            IfDstIdxValidxSI      : in  std_ulogic;
            SchedulerAckxDI       : in  std_ulogic_vector((C_SCALP_SCHEDULER_ACK_VECTOR_SIZE - 1) downto 0);
            BookingVectorxDO      : out std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
            BookingVectorValidxSO : out std_ulogic);
    end component scalp_booking_vector;

    component scalp_tx_fanout is
        generic (
            C_SCALP_NEIGHBORS_VECTOR_SIZE : integer range 0 to 255);
        port (
            RXAxi4M2SLinkxDI : in  t_axi4m2s;
            RXAxi4S2MLinkxDO : out t_axi4s2m;
            TXAxi4M2SLinkxDO : out t_axi4m2s_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
            TXAxi4S2MLinkxDI : in  t_axi4s2m_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0));
    end component scalp_tx_fanout;

    -- Signals
    -- Scalp Router Interface
    signal RXAxi4m2sIfxD        : t_axi4m2s                                                           := C_NO_AXI4_M2S;
    signal RXAxi4s2mIfxD        : t_axi4s2m                                                           := C_NO_AXI4_S2M;
    signal TXAxi4m2sIfxD        : t_axi4m2s                                                           := C_NO_AXI4_M2S;
    signal TXAxi4s2mIfxD        : t_axi4s2m                                                           := C_NO_AXI4_S2M;
    signal RXAxi4m2sLinksxD     : t_axi4m2s_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0)      := (others => C_NO_AXI4_M2S);
    signal RXAxi4s2mLinksxD     : t_axi4s2m_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0)      := (others => C_NO_AXI4_S2M);
    signal TXAxi4m2sLinksxD     : t_axi4m2s_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0)      := (others => C_NO_AXI4_M2S);
    signal TXAxi4s2mLinksxD     : t_axi4s2m_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0)      := (others => C_NO_AXI4_S2M);
    signal BookingVectorInxD    : std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0)     := (others => '0');
    signal BookingVectorOutxD   : std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0)     := (others => '0');
    signal SchedulerAckInxD     : std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0)     := (others => '0');
    signal SchedulerAckOutxD    : std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0)     := (others => '0');
    signal QoSVectorxD          : t_scalp_qos_vector((C_SCALP_INTERFACE_VECTOR_SIZE - 1) downto 0)    := (others => C_SCALP_NO_QOS);
    -- Scalp TX Side
    signal DstRouterNetAddrxD   : t_scalp_netaddr                                                     := C_3D_MIN_SCALP_NETADDR;
    signal RouterNetAddrValidxS : std_ulogic                                                          := '0';
    signal BookingVectorValidxS : std_ulogic                                                          := '0';
    signal SchedulerAckxD       : std_ulogic_vector((C_SCALP_SCHEDULER_ACK_VECTOR_SIZE - 1) downto 0) := (others => '0');
    signal TXAxi4M2SLinkxD      : t_axi4m2s                                                           := C_NO_AXI4_M2S;
    signal TXAxi4S2MLinkxD      : t_axi4s2m                                                           := C_NO_AXI4_S2M;
    -- Scalp RX Side
    -- Scalp Router Core
    signal LocRouterNetAddrxD   : t_scalp_netaddr                                                     := C_3D_MIN_SCALP_NETADDR;
    signal IfDstNumxD           : integer range 0 to (C_SCALP_INTERFACE_VECTOR_SIZE - 1)              := 0;
    signal IfNumValidxS         : std_ulogic                                                          := '0';
    -- Scalp Scheduler Ack
    -- Scalp Matrix Index
    signal IfSrcNumxD           : integer range 0 to (C_SCALP_INTERFACE_VECTOR_SIZE - 1)              := C_SCALP_INTERFACE_SOURCE_NUMBER;
    signal IfDstIdxxD           : integer range 0 to (C_SCALP_INTERFACE_VECTOR_SIZE - 1)              := 0;
    signal IfDstIdxValidxS      : std_ulogic                                                          := '0';
    -- Scalp Booking Vector
    -- Scalp TX Fanout
    -- Scalp Axis Delay
    signal DTXAxi4M2SLinkxD     : t_axi4m2s                                                           := C_NO_AXI4_M2S;
    signal DTXAxi4S2MLinkxD     : t_axi4s2m                                                           := C_NO_AXI4_S2M;

begin  -- architecture rtl

    EntityIOxB : block is
    begin  -- block EntityIOxB
        LocNetAddrxAS       : LocRouterNetAddrxD <= LocNetAddrxDI;
        RXAxi4m2sIfxAS      : RXAxi4m2sIfxD      <= RXAxi4m2sIfxDI;
        RXAxi4s2mIfxAS      : RXAxi4s2mIfxDO     <= RXAxi4s2mIfxD;
        TXAxi4m2sIfxAS      : TXAxi4m2sIfxDO     <= TXAxi4m2sIfxD;
        TXAxi4s2mIfxAS      : TXAxi4s2mIfxD      <= TXAxi4s2mIfxDI;
        RXAxi4m2sLinksxAS   : RXAxi4m2sLinksxD   <= RXAxi4m2sLinksxDI;
        RXAxi4s2mLinksxAS   : RXAxi4s2mLinksxDO  <= RXAxi4s2mLinksxD;
        TXAxi4m2sLinksxAS   : TXAxi4m2sLinksxDO  <= TXAxi4m2sLinksxD;
        TXAxi4s2mLinksxAS   : TXAxi4s2mLinksxD   <= TXAxi4s2mLinksxDI;
        BookingVectorInxAS  : BookingVectorInxD  <= BookingVectorxDI;
        BookingVectorOutxAS : BookingVectorxDO   <= BookingVectorOutxD;
        SchedulerAckInxAS   : SchedulerAckInxD   <= SchedulerAckxDI;
        SchedulerAckOutxAS  : SchedulerAckxDO    <= SchedulerAckOutxD;
        QoSVectorxAS        : QoSVectorxD        <= QoSVectorxDI;
    end block EntityIOxB;

    ScalpTXSidexI : entity work.scalp_tx_side
        generic map (
            C_SCALP_SCHEDULER_ACK_VECTOR_SIZE => C_SCALP_SCHEDULER_ACK_VECTOR_SIZE)
        port map (
            SysClkxCI             => SysClkxCI,
            SysRstxRNAI           => SysRstxRNAI,
            DstRouterNetAddrxDO   => DstRouterNetAddrxD,
            RouterNetAddrValidxSO => RouterNetAddrValidxS,
            BookingVectorValidxSI => BookingVectorValidxS,
            -- SchedulerAckxDI       => SchedulerAckxD,
            RXAxi4M2SLinkxDI      => RXAxi4m2sIfxD,
            RXAxi4S2MLinkxDO      => RXAxi4s2mIfxD,
            TXAxi4M2SLinkxDO      => TXAxi4M2SLinkxD,
            TXAxi4S2MLinkxDI      => TXAxi4S2MLinkxD);

    ScalpRXSidexI : entity work.scalp_rx_side
        generic map (
            C_SCALP_NEIGHBORS_VECTOR_SIZE => C_SCALP_NEIGHBORS_VECTOR_SIZE,
            C_SCALP_SCHEDULER_STRATEGY    => C_SCALP_SCHEDULER_STRATEGY,
            C_SCALP_SCHEDULER_OUTPUT_REG  => C_SCALP_SCHEDULER_OUTPUT_REG)
        port map (
            SysClkxCI            => SysClkxCI,
            SysRstxRNAI          => SysRstxRNAI,
            SchedulerAckxDO      => SchedulerAckOutxD,
            BookingVectorxDI     => BookingVectorInxD,
            TXAxi4M2SLinkxDO     => TXAxi4m2sIfxD,
            TXAxi4S2MLinkxDI     => TXAxi4s2mIfxD,
            RXAxi4M2SLinkxDI     => RXAxi4m2sLinksxD,
            RXAxi4S2MLinkxDO     => RXAxi4s2mLinksxD,
            BookingIndexxDO      => open,
            BookingIndexValidxSO => open);

    ScalpRouterCorexI : entity work.scalp_router_core
        generic map (
            C_SCALP_INTERFACE_VECTOR_SIZE => C_SCALP_INTERFACE_VECTOR_SIZE)
        port map (
            SysClkxCI             => SysClkxCI,
            SysRstxRNAI           => SysRstxRNAI,
            LocRouterNetAddrxDI   => LocRouterNetAddrxD,
            DstRouterNetAddrxDI   => DstRouterNetAddrxD,
            RouterNetAddrValidxSI => RouterNetAddrValidxS,
            IfDstNumxDO           => IfDstNumxD,
            IfNumValidxSO         => IfNumValidxS,
            QoSVectorxDI          => QoSVectorxD);

    ScalpSchedulerAckxI : entity work.scalp_scheduler_ack
        generic map (
            C_SCALP_NEIGHBORS_VECTOR_SIZE     => C_SCALP_NEIGHBORS_VECTOR_SIZE,
            C_SCALP_SCHEDULER_ACK_VECTOR_SIZE => C_SCALP_SCHEDULER_ACK_VECTOR_SIZE)
        port map (
            SchedulerAckxDI => SchedulerAckInxD,
            SchedulerAckxDO => SchedulerAckxD);

    ScalpMatrixIndexxI : entity work.scalp_matrix_index
        generic map (
            C_SCALP_INTERFACE_VECTOR_SIZE => C_SCALP_INTERFACE_VECTOR_SIZE)
        port map (
            IfSrcNumxDI      => IfSrcNumxD,
            IfDstNumxDI      => IfDstNumxD,
            IfNumValidxSI    => IfNumValidxS,
            IfDstIdxxDO      => IfDstIdxxD,
            IfDstIdxValidxSO => IfDstIdxValidxS);

    ScalpBookingVectorxI : entity work.scalp_booking_vector
        generic map (
            C_SCALP_INTERFACE_VECTOR_SIZE     => C_SCALP_INTERFACE_VECTOR_SIZE,
            C_SCALP_NEIGHBORS_VECTOR_SIZE     => C_SCALP_NEIGHBORS_VECTOR_SIZE,
            C_SCALP_SCHEDULER_ACK_VECTOR_SIZE => C_SCALP_SCHEDULER_ACK_VECTOR_SIZE)
        port map (
            SysClkxCI             => SysClkxCI,
            SysRstxRNAI           => SysRstxRNAI,
            IfDstIdxxDI           => IfDstIdxxD,
            IfDstIdxValidxSI      => IfDstIdxValidxS,
            SchedulerAckxDI       => SchedulerAckxD,
            BookingVectorxDO      => BookingVectorOutxD,
            BookingVectorValidxSO => BookingVectorValidxS);

    ScalpTXFanoutxI : entity work.scalp_tx_fanout
        generic map (
            C_SCALP_NEIGHBORS_VECTOR_SIZE => C_SCALP_NEIGHBORS_VECTOR_SIZE)
        port map (
            RXAxi4M2SLinkxDI => TXAxi4M2SLinkxD,
            RXAxi4S2MLinkxDO => TXAxi4S2MLinkxD,
            TXAxi4M2SLinkxDO => TXAxi4m2sLinksxD,
            TXAxi4S2MLinkxDI => TXAxi4s2mLinksxD);

end architecture rtl;
