----------------------------------------------------------------------------------
--                                 _             _
--                                | |_  ___ _ __(_)__ _
--                                | ' \/ -_) '_ \ / _` |
--                                |_||_\___| .__/_\__,_|
--                                         |_|
--
----------------------------------------------------------------------------------
--
-- Company: hepia
-- Author: Joachim Schmidt <joachim.schmidt@hesge.ch>
--
-- Module Name: tb_scalp_booking_vector.vhd
-- Target Device: SCALP xc7z015clg485-2
-- Tool version: 2019.1
-- Description: Testbench for scalp_booking_vector
--
-- Last update: 2020-02-13
--
---------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.uniform;
use ieee.math_real.floor;

--use ieee.std_logic_unsigned.all;
--use ieee.std_logic_arith.all;
--use ieee.std_logic_misc.all;

--library UNISIM;
--use UNISIM.VCOMPONENTS.all;

library work;
use work.scalp_utility.all;

entity tb_scalp_booking_vector is

end entity tb_scalp_booking_vector;

architecture testbench of tb_scalp_booking_vector is

    constant C_SCALP_INTERFACE_VECTOR_SIZE     : integer range 0 to 255                                          := 7;
    constant C_SCALP_NEIGHBORS_VECTOR_SIZE     : integer range 0 to 255                                          := 6;
    constant C_SCALP_SCHEDULER_ACK_VECTOR_SIZE : integer range 0 to 255                                          := 1;
    constant C_CLK_PERIOD                      : time                                                            := 10 ns;
    constant C_SCALP_BOOKING_VECTOR_EMPTY      : std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0) := (others => '0');

    component scalp_matrix_index is
        generic (
            C_SCALP_INTERFACE_VECTOR_SIZE : integer range 0 to 255);
        port (
            IfSrcNumxDI      : in  integer range 0 to (C_SCALP_INTERFACE_VECTOR_SIZE - 1);
            IfDstNumxDI      : in  integer range 0 to (C_SCALP_INTERFACE_VECTOR_SIZE - 1);
            IfNumValidxSI    : in  std_ulogic;
            IfDstIdxxDO      : out integer range 0 to (C_SCALP_INTERFACE_VECTOR_SIZE - 1);
            IfDstIdxValidxSO : out std_ulogic);
    end component scalp_matrix_index;

    component scalp_booking_vector is
        generic (
            C_SCALP_INTERFACE_VECTOR_SIZE     : integer range 0 to 255;
            C_SCALP_NEIGHBORS_VECTOR_SIZE     : integer range 0 to 255;
            C_SCALP_SCHEDULER_ACK_VECTOR_SIZE : integer range 0 to 255);
        port (
            SysClkxCI             : in  std_ulogic;
            SysRstxRNAI           : in  std_ulogic;
            IfDstIdxxDI           : in  integer range 0 to (C_SCALP_INTERFACE_VECTOR_SIZE - 1);
            IfDstIdxValidxSI      : in  std_ulogic;
            SchedulerAckxDI       : in  std_ulogic_vector((C_SCALP_SCHEDULER_ACK_VECTOR_SIZE - 1) downto 0);
            BookingVectorxDO      : out std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
            BookingVectorValidxSO : out std_ulogic);
    end component scalp_booking_vector;

    signal SysClkxC             : std_ulogic                                                          := '0';
    signal SysRstxRNA           : std_ulogic                                                          := '0';
    -- Scalp Matrix Index
    signal IfSrcNumxD           : integer range 0 to (C_SCALP_INTERFACE_VECTOR_SIZE - 1)              := 0;
    signal IfDstNumxD           : integer range 0 to (C_SCALP_INTERFACE_VECTOR_SIZE - 1)              := 0;
    signal IfNumValidxS         : std_ulogic                                                          := '0';
    signal IfDstIdxxD           : integer range 0 to (C_SCALP_INTERFACE_VECTOR_SIZE - 1)              := 0;
    signal IfDstIdxValidxS      : std_ulogic                                                          := '0';
    -- Scalp Booking Vector
    signal SchedulerAckxD       : std_ulogic_vector((C_SCALP_SCHEDULER_ACK_VECTOR_SIZE - 1) downto 0) := (others => '0');
    signal BookingVectorxD      : std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0)     := (others => '0');
    signal BookingVectorValidxS : std_ulogic                                                          := '0';
    -- Process Signals
    signal EndScalpMatIdxxS     : std_ulogic                                                          := '0';
    signal RandomWaitTimexD     : integer                                                             := 10;

begin  -- architecture testbench

    SysClkxAS : SysClkxC <= not SysClkxC after (C_CLK_PERIOD / 2);

    ScalpMatrixIndexxI : entity work.scalp_matrix_index
        generic map (
            C_SCALP_INTERFACE_VECTOR_SIZE => C_SCALP_INTERFACE_VECTOR_SIZE)
        port map (
            IfSrcNumxDI      => IfSrcNumxD,
            IfDstNumxDI      => IfDstNumxD,
            IfNumValidxSI    => IfNumValidxS,
            IfDstIdxxDO      => IfDstIdxxD,
            IfDstIdxValidxSO => IfDstIdxValidxS);

    ScalpBookingVectorxI : entity work.scalp_booking_vector
        generic map (
            C_SCALP_INTERFACE_VECTOR_SIZE     => C_SCALP_INTERFACE_VECTOR_SIZE,
            C_SCALP_NEIGHBORS_VECTOR_SIZE     => C_SCALP_NEIGHBORS_VECTOR_SIZE,
            C_SCALP_SCHEDULER_ACK_VECTOR_SIZE => C_SCALP_SCHEDULER_ACK_VECTOR_SIZE)
        port map (
            SysClkxCI             => SysClkxC,
            SysRstxRNAI           => SysRstxRNA,
            IfDstIdxxDI           => IfDstIdxxD,
            IfDstIdxValidxSI      => IfDstIdxValidxS,
            SchedulerAckxDI       => SchedulerAckxD,
            BookingVectorxDO      => BookingVectorxD,
            BookingVectorValidxSO => BookingVectorValidxS);

    ResetxP : process is
    begin  -- process ResetxP
        wait for C_CLK_PERIOD;
        SysRstxRNA <= '1';
        wait;
    end process ResetxP;

    RandomWaitTimexP : process is

        variable Seed1xD, Seed2xD : positive;
        variable RandxD           : real;
        variable RangeOfRandxD    : real := 20.0;

    begin  -- process RandomWaitTimexP

        report "Random wait Time is started" severity note;

        RVTLoopxAS : loop
            exit RVTLoopxAS when (EndScalpMatIdxxS = '1');
            uniform(Seed1xD, Seed2xD, RandxD);
            RandomWaitTimexD <= integer(RandxD * RangeOfRandxD) + 10;
            -- report "RandomWaitTimexD = " & integer'image(RandomWaitTimexD) severity note;
            wait for C_CLK_PERIOD;
        end loop;

        report "Random wait Time is finished" severity note;

        wait;
    end process RandomWaitTimexP;

    ScalpRouterValidxP : process is
    begin  -- process ScalpRouterValidxP

        report "Scalp Router Valid is started" severity note;

        SRVLoopxAS : loop
            exit SRVLoopxAS when (EndScalpMatIdxxS = '1');
            wait for RandomWaitTimexD * C_CLK_PERIOD;
            IfNumValidxS <= '1';
            wait for C_CLK_PERIOD;
            IfNumValidxS <= '0';
        end loop;

        report "Scalp Router Valid is finished" severity note;

        wait;
    end process ScalpRouterValidxP;


    CheckScalpMatrixIndexxP : process is
    begin  -- process CheckScalpMatrixIndexxP

        wait for 2 * C_CLK_PERIOD;

        report "Scalp Matrix Index checking is started" severity note;

        for i in 0 to (C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) loop
            IfSrcNumxD <= i;
            for j in 0 to (C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) loop
                IfDstNumxD <= j;

                wait until IfNumValidxS = '1';

                report "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@" severity note;
                report "Iteration i = " & integer'image(i) & ", j = " & integer'image(j) severity note;
                report "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@" severity note;

            end loop;  -- j
        end loop;  -- i

        EndScalpMatIdxxS <= '1';

        report "Scalp Matrix Index checking is finished" severity note;

        wait;
    end process CheckScalpMatrixIndexxP;

    CheckScalpBookingVectorxP : process is
    begin  -- process CheckScalpBookingVectorxP

        wait for 2 * C_CLK_PERIOD;

        report "Scalp Booking Vector checking is started" severity note;

        CSBVxAS : loop
            exit CSBVxAS when (EndScalpMatIdxxS = '1');
            wait until (BookingVectorxD /= C_SCALP_BOOKING_VECTOR_EMPTY);
            SchedulerAckxD(0) <= '1';
            wait for 5 * C_CLK_PERIOD;
            SchedulerAckxD(0) <= '0';
        end loop;

        report "Scalp Booking Vector checking is finished" severity note;

        wait;
    end process CheckScalpBookingVectorxP;

end architecture testbench;
